@@ -25,9 +25,9 @@
 //! C D
 //! ```
 //!
-//! This module fixes these sliders by sliding novel region regions
-//! forwards or backwards when the before and after nodes are the same
-//! (B in this example).
+//! This module fixes these cases. It identifies situations where we
+//! can change which item is marked as novel (e.g. either `B` in the
+//! example above) whilst still showing a valid, minimal diff.
 
 use crate::{
     diff::changes::{insert_deep_novel, insert_deep_unchanged, ChangeKind::*, ChangeMap},
@@ -60,9 +60,10 @@ fn prefer_outer_delimiter(language: guess_language::Language) -> bool {
         // JSON and TOML are like Lisp: the outer delimiter in an array object
         // is the most relevant.
         Json | Toml | Hcl => true,
-        // It's probably the case that outer delimiters are used more
-        // frequently than inner delimiters in SQl. (foo = 1 OR bar = 2)
-        // is more likely than foo(1).
+        // It's probably the case that outer delimiters
+        // (e.g. grouping) are used more frequently than inner
+        // delimiters in SQL. `(foo = 1 OR bar = 2)` is more likely
+        // than `foo(1)`.
         Sql => true,
         // For everything else, prefer the inner delimiter. These
         // languages have syntax like `foo(bar)` or `foo[bar]` where
@@ -306,6 +307,9 @@ fn push_unchanged_to_ancestor<'a>(
     }
 }
 
+/// For every sequence of novel nodes, if it's a potential slider,
+/// change which nodes are marked as novel if it produces a sequence
+/// of nodes that are closer together.
 fn fix_sliders<'a>(nodes: &[&'a Syntax<'a>], change_map: &mut ChangeMap<'a>) {
     for (region_start, region_end) in novel_regions_after_unchanged(nodes, change_map) {
         slide_to_prev_node(nodes, change_map, region_start, region_end);
@@ -315,6 +319,8 @@ fn fix_sliders<'a>(nodes: &[&'a Syntax<'a>], change_map: &mut ChangeMap<'a>) {
     }
 }
 
+/// Return the start and end indexes of sequences of novel nodes that
+/// occur after unchanged nodes.
 fn novel_regions_after_unchanged<'a>(
     nodes: &[&'a Syntax<'a>],
     change_map: &ChangeMap<'a>,
@@ -363,6 +369,8 @@ fn novel_regions_after_unchanged<'a>(
         .collect()
 }
 
+/// Return the start and end indexes of sequences of novel nodes that
+/// occur before unchanged nodes.
 fn novel_regions_before_unchanged<'a>(
     nodes: &[&'a Syntax<'a>],
     change_map: &ChangeMap<'a>,
@@ -422,24 +430,24 @@ fn is_novel_deep<'a>(node: &Syntax<'a>, change_map: &ChangeMap<'a>) -> bool {
     }
 }
 
-fn is_unchanged_deep<'a>(node: &Syntax<'a>, change_map: &ChangeMap<'a>) -> bool {
-    match node {
-        List { children, .. } => {
-            if !matches!(change_map.get(node), Some(Unchanged(_))) {
-                return false;
-            }
-            for child in children {
-                if !is_unchanged_deep(child, change_map) {
-                    return false;
-                }
-            }
-
-            true
-        }
-        Atom { .. } => matches!(change_map.get(node), Some(Unchanged(_))),
-    }
-}
-
+/// If the previous node is unchanged, matches the end of the region,
+/// and has a smaller text distance, mark it as novel.
+///
+/// ```text
+/// x UNCHANGED
+/// y NOVEL <- start_idx
+///
+/// x NOVEL <- end_idx
+/// ```
+///
+/// After this function:
+///
+/// ```text
+/// x NOVEL
+/// y NOVEL
+///
+/// x UNCHANGED
+/// ```
 fn slide_to_prev_node<'a>(
     nodes: &[&'a Syntax<'a>],
     change_map: &mut ChangeMap<'a>,
@@ -466,30 +474,51 @@ fn slide_to_prev_node<'a>(
     let distance_to_last = distance_between(before_last_node, last_node);
 
     if distance_to_before_start <= distance_to_last {
-        // Deep checks walk the whole tree, so do these last.
-        if !is_unchanged_deep(before_start_node, change_map) {
-            return;
-        }
+        let opposite = match change_map
+            .get(before_start_node)
+            .expect("Node changes should be set")
+        {
+            Unchanged(n) => {
+                if before_start_node.content_id() != n.content_id() {
+                    return;
+                }
+                n
+            }
+            _ => {
+                return;
+            }
+        };
+
         for node in &nodes[start_idx..=end_idx] {
             if !is_novel_deep(node, change_map) {
                 return;
             }
         }
 
-        let opposite = match change_map
-            .get(before_start_node)
-            .expect("Node changes should be set")
-        {
-            Unchanged(n) => n,
-            _ => unreachable!(),
-        };
-
         insert_deep_novel(before_start_node, change_map);
         insert_deep_unchanged(last_node, opposite, change_map);
         insert_deep_unchanged(opposite, last_node, change_map);
     }
 }
 
+/// If the next node is unchanged, matches the beginning of the region,
+/// and has a smaller text distance, mark it as novel.
+///
+/// ```text
+/// x NOVEL <- start_idx
+///
+/// y NOVEL <- end_idx
+/// x UNCHANGED
+/// ```
+///
+/// After this function:
+///
+/// ```text
+/// x UNCHANGED
+///
+/// y NOVEL
+/// x NOVEL
+/// ```
 fn slide_to_next_node<'a>(
     nodes: &[&'a Syntax<'a>],
     change_map: &mut ChangeMap<'a>,
@@ -516,24 +545,26 @@ fn slide_to_next_node<'a>(
     let distance_to_after_last = distance_between(last_node, after_last_node);
 
     if distance_to_after_last < distance_to_start {
-        // Deep checks walk the whole tree, so do these last.
-        if !is_unchanged_deep(after_last_node, change_map) {
-            return;
-        }
+        let opposite = match change_map
+            .get(after_last_node)
+            .expect("Node changes should be set")
+        {
+            Unchanged(n) => {
+                if after_last_node.content_id() != n.content_id() {
+                    return;
+                }
+                n
+            }
+            _ => {
+                return;
+            }
+        };
         for node in &nodes[start_idx..=end_idx] {
             if !is_novel_deep(node, change_map) {
                 return;
             }
         }
 
-        let opposite = match change_map
-            .get(after_last_node)
-            .expect("Node changes should be set")
-        {
-            Unchanged(n) => n,
-            _ => unreachable!(),
-        };
-
         insert_deep_unchanged(start_node, opposite, change_map);
         insert_deep_unchanged(opposite, start_node, change_map);
         insert_deep_novel(after_last_node, change_map);
@@ -714,6 +745,7 @@ mod tests {
         assert_eq!(change_map.get(lhs[1]), Some(Novel));
         assert_eq!(change_map.get(lhs[2]), Some(Novel));
     }
+
     #[test]
     fn test_slider_two_steps() {
         let arena = Arena::new();
@@ -736,4 +768,46 @@ mod tests {
         assert_eq!(change_map.get(rhs[2]), Some(Novel));
         assert_eq!(change_map.get(rhs[3]), Some(Unchanged(rhs[0])));
     }
+
+    /// If a list is partially unchanged but contains some novel
+    /// children, we should not slide it.
+    #[test]
+    fn test_slider_partially_unchanged() {
+        let arena = Arena::new();
+        let config = from_language(guess_language::Language::EmacsLisp);
+
+        let lhs = parse(&arena, "(A B) X \n (A B)", &config, false);
+        let rhs = parse(&arena, "((novel) A B)", &config, false);
+        init_all_info(&lhs, &rhs);
+
+        let lhs_first_list_children = match lhs[0] {
+            List { children, .. } => children,
+            Atom { .. } => unreachable!(),
+        };
+        let rhs_first_list_children = match rhs[0] {
+            List { children, .. } => children,
+            Atom { .. } => unreachable!(),
+        };
+
+        let mut change_map = ChangeMap::default();
+        change_map.insert(lhs[0], Unchanged(rhs[0]));
+        change_map.insert(lhs[1], Novel);
+        insert_deep_novel(lhs[2], &mut change_map);
+
+        change_map.insert(
+            lhs_first_list_children[0],
+            Unchanged(rhs_first_list_children[1]),
+        );
+        change_map.insert(
+            lhs_first_list_children[1],
+            Unchanged(rhs_first_list_children[2]),
+        );
+
+        fix_all_sliders(guess_language::Language::EmacsLisp, &lhs, &mut change_map);
+        assert_eq!(
+            change_map.get(lhs[2]),
+            Some(Novel),
+            "The novel node at the end should be unaffected"
+        );
+    }
 }
