@@ -18,6 +18,10 @@
 // Purely stylistic, and ignores whether there are explanatory
 // comments in the if/else.
 #![allow(clippy::bool_to_int_with_if)]
+// Good practice in general, but a necessary evil for Syntax. Its Hash
+// implementation does not consider the mutable fields, so it is still
+// correct.
+#![allow(clippy::mutable_key_type)]
 
 mod constants;
 mod diff;
@@ -116,6 +120,7 @@ fn main() {
         Mode::DumpSyntax {
             path,
             language_override,
+            ignore_comments,
         } => {
             let path = Path::new(&path);
             let bytes = read_or_die(path);
@@ -126,8 +131,7 @@ fn main() {
                 Some(lang) => {
                     let ts_lang = tsp::from_language(lang);
                     let arena = Arena::new();
-                    // TODO: respect ignore_comments
-                    let ast = tsp::parse(&arena, &src, &ts_lang, false);
+                    let ast = tsp::parse(&arena, &src, &ts_lang, ignore_comments);
                     init_all_info(&ast, &[]);
                     println!("{:#?}", ast);
                 }
