@@ -56,6 +56,7 @@ pub enum Language {
     Php,
     Python,
     Qml,
+    R,
     Ruby,
     Rust,
     Scala,
@@ -109,6 +110,7 @@ pub fn language_name(language: Language) -> &'static str {
         Php => "PHP",
         Python => "Python",
         Qml => "QML",
+        R => "R",
         Ruby => "Ruby",
         Rust => "Rust",
         Scala => "Scala",
@@ -201,6 +203,7 @@ pub const LANG_EXTENSIONS: &[(Language, &[&str])] = &[
     (Php, &["php"]),
     (Python, &["py", "py3", "pyi", "bzl"]),
     (Qml, &["qml"]),
+    (R, &["R", "r", "rd", "rsx"]),
     (Ruby, &["rb", "builder", "spec", "rake"]),
     (Rust, &["rs"]),
     (Scala, &["scala", "sbt", "sc"]),
@@ -293,6 +296,7 @@ pub const LANG_FILE_NAMES: &[(Language, &[&str])] = &[
         ],
     ),
     (Python, &["TARGETS", "BUCK", "DEPS"]),
+    (R, &[".Rprofile", "expr-dist"]),
     (Ruby, &["Gemfile", "Rakefile"]),
     (
         Toml,
@@ -407,6 +411,7 @@ fn from_shebang(src: &str) -> Option<Language> {
                     "ocaml" | "ocamlrun" | "ocamlscript" => return Some(OCaml),
                     "perl" => return Some(Perl),
                     "python" | "python2" | "python3" => return Some(Python),
+                    "Rscript" => return Some(R),
                     "ruby" | "macruby" | "rake" | "jruby" | "rbx" => return Some(Ruby),
                     "swift" => return Some(Swift),
                     "deno" | "ts-node" => return Some(TypeScript),
